@@ -3,6 +3,10 @@ function (f, X0, ..., y0=NULL, n = 200, L = 1000, delta = 0.003, nchains = 5,
     T.mult = 1.5, lb = NULL, ub = NULL, maxleap=0.95, finetune = FALSE,
     Tinclude = nchains, nswaps = choose(nchains, 2), graph = FALSE)
 {
+    negInf <- function(X)
+      as.double(apply(X, 2, function(x) 101*min(x)-100*max(x)))
+    posInf <- function(X)
+      as.double(apply(X, 2, function(x) 101*max(x)-100*min(x)))
     fn <- function(par) f(par, ...)
     X <- as.matrix(X0)
     yfinal <- Xfinal <- NULL
@@ -21,10 +25,13 @@ function (f, X0, ..., y0=NULL, n = 200, L = 1000, delta = 0.003, nchains = 5,
             stop(paste(sep = "", "Duplicate rows (", i, " & ", 
                 j, ") in X not allowed"))
     BOUNDS <- FALSE
-    if ((!is.null(lb))  || (!is.null(ub)))
-            BOUNDS <- TRUE
-    if(is.null(lb)) lb <- rep(-Inf,d)
-    if(is.null(ub)) ub <- rep(Inf,d)
+    if ((!is.null(lb))  || (!is.null(ub))) BOUNDS <- TRUE
+    if(is.null(lb)) lb <- rep(-Inf, d)
+    low <- lb
+    low[!(low>-Inf)] <- negInf(X)[!(low>-Inf)]
+    if(is.null(ub)) ub <- rep(Inf, d)
+    upp <- ub
+    upp[!(upp<Inf)] <- posInf(X)[!(upp<Inf)]
     if (BOUNDS) {
       if (!is.null(lb)) 
           if (length(lb) == 1) 
@@ -35,11 +42,7 @@ function (f, X0, ..., y0=NULL, n = 200, L = 1000, delta = 0.003, nchains = 5,
       if ((length(lb) != d) || (length(lb) != d)) 
           stop("lb, ub incorrect dimension")
       if (is.na(sum(lb + ub))) 
-          stop("hybrid.explore: NaN lb, ub")
-      cat("hybrid.explore: Placing bounds lb =\n")
-      cat(lb, "\n")
-      cat("hybrid.explore: Placing bounds ub =\n")
-      cat(ub, "\n")
+          stop("NaN lb, ub")
     }
     digits <- rep(NA, d)
     for (j in 1:d) {
@@ -56,15 +59,8 @@ function (f, X0, ..., y0=NULL, n = 200, L = 1000, delta = 0.003, nchains = 5,
         KEY.VARS <- rbind(KEY.VARS, pair)
     }
     if (graph) {
-        if(!exists(".Rpackage_DEMO")) {
-          Rpackage_DEMO <- .Rpackage_DEMO <- FALSE
-        } else {
-          Rpackage_DEMO <- .Rpackage_DEMO
-          assign(".Rpackage_DEMO", FALSE, envir=.GlobalEnv)
-        }
-        nrefresh.graph <- ceiling(10/nchains)
-        if(is.logical(Rpackage_DEMO))
-          if(Rpackage_DEMO) nrefresh.graph <- Inf
+        if(dim(X)[2] <= 2) nrefresh.graph <- Inf else
+          nrefresh.graph <- ceiling(10/nchains)
         graph.template <- .hybrid.template
         key.indx <- 1
         keyVars <- KEY.VARS[key.indx, ]
@@ -75,14 +71,9 @@ function (f, X0, ..., y0=NULL, n = 200, L = 1000, delta = 0.003, nchains = 5,
     if(!is.null(y0)) {
       y <- y0 
     } else {
-      cat(sep = "", "hybrid.explore: ", n0, " initial ", d, "-D points supplied for ", 
-          n, " Exploratory points\n")
-      cat("Leapfrogs: L =", L, "and delta =", delta, "\n")
       if (T.mult < 1) 
           stop("T.mult < 1 not allowed")
       y <- rep(NA, n0)
-      cat("Parallel Tempering:", nchains, "chains with T:", Temp, 
-          "\n")
       date1 <- date2 <- date()
       for (j in 1:n0) {
           functionOK <- FALSE
@@ -91,25 +82,20 @@ function (f, X0, ..., y0=NULL, n = 200, L = 1000, delta = 0.003, nchains = 5,
             OUTSIDE_BOUNDS <- FALSE
             if (BOUNDS) {
               for (i in 1:d) {
-                if (Xj[i] < lb[i]) {
+                if (Xj[i] < low[i]) {
                   OUTSIDE_BOUNDS <- TRUE
-                  Xj[i] <- lb[i]
+                  Xj[i] <- low[i]
                 }
-                if (Xj[i] > ub[i]) {
+                if (Xj[i] > upp[i]) {
                   OUTSIDE_BOUNDS <- TRUE
-                  Xj[i] <- ub[i]
+                  Xj[i] <- upp[i]
                 }
               }
             }
             if(OUTSIDE_BOUNDS) stop("X0 violates bounds supplied")
             date1 <- date()
-            cat("\n")
             y[j] <- fn(X[j,])
             date2 <- date()
-            cat(sep = "", "f(x", j, ") = ", y[j], " for x", j, 
-                ":\n")
-            cat("", X[j, ], "\n")
-            .show.time(date1, date2, graph = FALSE)
             functionOK <- TRUE
           })
           if(!functionOK) stop("f(x) failed")
@@ -137,10 +123,12 @@ function (f, X0, ..., y0=NULL, n = 200, L = 1000, delta = 0.003, nchains = 5,
     nacc <- 0
     refresh_count <- 1
     params <- rep(1, d+1)
+    mean.acceptance <- 0
     repeat {
-        cat("\n")
-        GPfit <- GProcess(X, y, params = params, request.functions = FALSE, finetune = finetune)
-        if(!GPfit$inverseOK) stop("hybrid.explore: GProcess inverse covariance matrix failed")
+        GPfit <- GProcess(X, y, params = params, request.functions = FALSE,
+                          finetune = finetune)
+        if(!GPfit$inverseOK)
+          stop("GProcess inverse covariance matrix failed")
         Sigma.inv <- GPfit$Sigma.inv
         params <- GPfit$params
         lsq <- params[1]
@@ -196,8 +184,6 @@ function (f, X0, ..., y0=NULL, n = 200, L = 1000, delta = 0.003, nchains = 5,
                   Temp[swap[1]] <- T2
                   Temp[swap[2]] <- T1
                   swaps.record <- c(swaps.record, 1)
-                  cat("hybrid.explore: Swap T =", T1, "with", 
-                    T2, "\n")
                 }
                 else {
                   swaps.record <- c(swaps.record, 0)
@@ -215,10 +201,6 @@ function (f, X0, ..., y0=NULL, n = 200, L = 1000, delta = 0.003, nchains = 5,
             p.C <- NULL
             sdf_x.C <- sigmaf(x.C)
             corf_x.C <- sdf_x.C/sqrt(lsq)
-            cat("\n")
-            cat(sep = "", "Leapfrog from x(", chain, ", T = ", 
-                Temp[chain], "):\n")
-            cat("", rnd(x.C), "\n")
             fx.C <- fx[chain]
             xp_Early <- NULL
             count <- 0
@@ -226,27 +208,33 @@ function (f, X0, ..., y0=NULL, n = 200, L = 1000, delta = 0.003, nchains = 5,
                 p.C <- rnorm(d)
                 xp_Early <- matrix(c(x.C, p.C, rep(0, d)), ncol = 3)
                 xOK <- FALSE
-                low <- lb
-                upp <- ub
                 try({
-                  xp_Early <- .Call("Leapfrog", X, y, x.C, p.C, 
-                    L, delta, GPfit$Sigma.inv, params, Temp[chain], sqrt(lsq)*maxleap, low, upp)
+                  out <- .C("Leap", as.double(X), as.integer(dim(X)[1]),
+                    as.integer(d), as.double(y), as.double(x.C),
+                    as.double(p.C), as.integer(L), as.double(delta),
+                    as.double(GPfit$Sigma.inv), as.double(params),
+                    as.double(Temp[chain]), as.double(sqrt(lsq)*maxleap),
+                    as.double(low), as.double(upp),
+                    as.double(rep(0,n)), as.double(rep(0,n)),
+                    as.double(rep(0,d)), as.double(rep(0,d)),
+                    xp_Early = as.double(xp_Early) )
+                  xp_Early <- out$xp_Early
+                  dim(xp_Early) <- c(d,3)
                   xOK <- TRUE
                 })
-                if (is.na(sum(xp_Early))) 
-                  xOK <- FALSE
-                if (xOK) 
-                  break
-                cat("hybrid.explore: Proposed x failed\n")
+                if (is.na(sum(xp_Early))) xOK <- FALSE
+                if (xOK) break
+                message("hybrid.explore: Proposed x failed")
                 count <- count + 1
                 if (count == 10) {
                   for (remv in 1:nchains) {
                     X <- X[-length(y), ]
                     y <- y[-length(y)]
                   }
-                  GPfit <- GProcess(X, y, params = params, request.functions = TRUE)
+                  GPfit <- GProcess(X, y, params = params,
+                    request.functions = TRUE)
                   if(!GPfit$inverseOK)
-                    stop("hybrid.explore: GProcess inverse covariance matrix failed")
+                    stop("GProcess inverse covariance matrix failed")
                   Sigma.inv <- GPfit$Sigma.inv
                   params <- GPfit$params
                   lsq <- params[1]
@@ -272,18 +260,15 @@ function (f, X0, ..., y0=NULL, n = 200, L = 1000, delta = 0.003, nchains = 5,
                 }
             }
             p.P <- out$p
-            cat(sep = "", "Proposed x(", chain, ", T =", 
-              Temp[chain], "):\n")
-            cat("", rnd(x.P), "\n")
             sdf_x.P <- sigmaf(x.P)
             corf_x.P <- sdf_x.P/sqrt(lsq)
             if(as.integer(count10/10) == count10/10) MAX10 <- 0
             if(corf_x.P > MAX10) MAX10 <- corf_x.P
             if (out$Early < 0) {
-              cat("Early stop: sd(f(x))/lambda =", corf_x.P, "where maxleap =", maxleap, "\n")
+              message(paste(sep="","Early stop with sd(f(x))/lambda = ",
+                  round(corf_x.P,5), " where maxleap = ", maxleap))
             }
             else {
-              cat("Full Leapfrog: sd(f(x))/lambda =", corf_x.P, "where maxleap =", maxleap, "\b)\n")
               if (corf_x.P > maxleap) 
                 stop(paste("Caution: Leapfrog gave sigmaf =", 
                   out$Early/sqrt(lsq), "when sigmaf(x)/lambda =", corf_x.P))
@@ -297,12 +282,11 @@ function (f, X0, ..., y0=NULL, n = 200, L = 1000, delta = 0.003, nchains = 5,
                 lty = 2, col = clr)
             }
             if(OUTSIDE_BOUNDS) {
-              cat("T =", Temp[chain], "\b: Ignore point out of bounds\n")
+              message(paste(sep="","T = ", Temp[chain],
+                            ": Ignore point out of bounds"))
               fx.P <- alpha <- -Inf
             } else {
-              cat("Leapfrog: Calculate true f(x) for proposal.\n")
               date1 <- date()
-              cat("\n")
               fx.P <- fn(x.P)
               date2 <- date()
               if (is.na(fx.P)) 
@@ -310,8 +294,6 @@ function (f, X0, ..., y0=NULL, n = 200, L = 1000, delta = 0.003, nchains = 5,
               if (fx.P == Inf) 
                 stop("Infinite f(x) not allowed")
               fcalls <- fcalls + 1
-              cat("f(x) =", fx.P, "leap from f(x)", fx.C, "\n")
-              .show.time(date1, date2, graph)
               H_xp.C <- -fx.C - sdf_x.C + W(p.C)
               H_xp.P <- -fx.P - sdf_x.P + W(p.P)
               alpha <- min(((-H_xp.P + H_xp.C)/Temp[chain]), 0)
@@ -326,25 +308,21 @@ function (f, X0, ..., y0=NULL, n = 200, L = 1000, delta = 0.003, nchains = 5,
                 yfinal <- c(yfinal, fx.P)
                 Taccept <- c(Taccept, path)
                 acceptance <- c(acceptance, 1)
+                if(dim(Xfinal)[1] %% floor(n/10) == 0)
+                  message(paste(sep="","hybrid.explore: ",
+                    round(100*dim(Xfinal)[1]/n),"% Complete with ",
+                    round(100*mean(acceptance),1),"% acceptance of proposals"))
               }
-              cat(sep = "", "ACCEPT x(", chain, ", T = ", 
-                Temp[chain], "):\n")
-              cat("", rnd(x.P), "\n")
-              if(path > Tinclude) cat("Point marked for later removal",
-                          "(Tinclude =", Tinclude, "\b)\n")
               if (graph) 
                 points(x.P[keyVars[1]], x.P[keyVars[2]], 
                   pch = 20, col = clr)
             }
             else {
-              cat(sep = "", "REJECT x.P (", chain, ", T=", Temp[chain], "):\n")
               if(path == 1) if(!OUTSIDE_BOUNDS) {
                 Xnext <- rbind(Xnext, x.P)
                 ynext <- c(ynext, fx.P)
                 Taccept <- c(Taccept, Tinclude+1)
-                cat("Primary chain point marked for later removal\n")
               }
-              cat("", rnd(x.P), "\n")
               if(path <= Tinclude) acceptance <- c(acceptance, 0)
               if (graph) if(!OUTSIDE_BOUNDS)
                 points(x.P[keyVars[1]], x.P[keyVars[2]], 
@@ -362,26 +340,14 @@ function (f, X0, ..., y0=NULL, n = 200, L = 1000, delta = 0.003, nchains = 5,
                 if(length(ynext) <= n) break
               }
             }
-            if (nchains >= 2) 
-              cat(sep="", "Swaps: ", round(10000 * mean(swaps.record))/100, "% acceptance\n")
             if (is.null(acceptance)) 
               mean.acceptance <- 0 else
               mean.acceptance <- mean(acceptance[ceiling(length(acceptance)/2):length(acceptance)])
-            cat(sep = "", "hybrid.explore: ",
-              round(10000 * mean.acceptance)/100, "% acceptance of required points\n")
-            cat("Leapfrog used: L =", L, ", delta =", delta, "\n")
-            cat("Present Gaussian process consists of", dim(Xnext)[1], "points\n")
-            cat("So far", sum(Taccept<=Tinclude), "/", n, "required points in",
-              Tinclude, "out of", nchains, "chains\n")
             if(!is.null(Xfinal)) if(dim(Xfinal)[1] >= n) break
             count10 <- count10+1
-            cat("sd(f(x))/lambda =", corf_x.P, "where maxleap =", maxleap, "\n")
-            cat("Maximum of 10 observed sd(f(x)) =", MAX10, "\n")
         }
         X <- Xnext
         y <- ynext
-        assign("hybrid.explore.out", list(X = Xfinal, y = yfinal, Xfull=X, yfull=y, acceptance = acceptance,
-            X0 = X0), envir = .GlobalEnv)
         if (is.null(acceptance)) 
             mean.acceptance <- NA
         else mean.acceptance <- mean(acceptance)
@@ -407,7 +373,7 @@ function (f, X0, ..., y0=NULL, n = 200, L = 1000, delta = 0.003, nchains = 5,
     y <- yfinal
     GPfit <- GProcess(X, y, params, request.functions = TRUE, finetune=TRUE)
     if(!GPfit$inverseOK)
-      cat("hybrid.explore: GProcess inverse covariance matrix failed\n")
+      message("hybrid.explore: GProcess inverse covariance matrix failed")
     details <- list(nchains = nchains, T.mult = T.mult, nswaps = nswaps, L = L, delta = delta,
         lb = lb, ub = ub, KEY.VARS = KEY.VARS)
     return(list(X = X, y = y, f = fn, maxleap=maxleap, function.calls = fcalls, details = details, GPfit = GPfit))
