@@ -3,6 +3,10 @@ function (Explore, n = 1000, replace.target = c(0, 1, 2),
     lb=NULL, ub=NULL, L = NULL, delta = NULL, nchains = NULL,
     T.mult = NULL, maxleap=NULL, r = 5, nswaps = NULL, graph = FALSE)
 {
+    negInf <- function(X)
+      as.double(apply(X, 2, function(x) 101*min(x)-100*max(x)))
+    posInf <- function(X)
+      as.double(apply(X, 2, function(x) 101*max(x)-100*min(x)))
     X <- Explore$X
     y <- Explore$y
     f <- Explore$f
@@ -27,8 +31,13 @@ function (Explore, n = 1000, replace.target = c(0, 1, 2),
     if(is.null(lb)) lb <- Explore$details$lb
     if(is.null(ub)) ub <- Explore$details$ub
     BOUNDS <- FALSE
-    if ((!is.null(lb)) || (!is.null(ub)))
-            BOUNDS <- TRUE
+    if ((!is.null(lb)) || (!is.null(ub))) BOUNDS <- TRUE
+    if(is.null(lb)) lb <- rep(-Inf, d)
+    low <- lb
+    low[!(low>-Inf)] <- negInf(X)[!(low>-Inf)]
+    if(is.null(ub)) ub <- rep(Inf, d)
+    upp <- ub
+    upp[!(upp<Inf)] <- posInf(X)[!(upp<Inf)]
     if(is.null(lb)) lb <- rep(-Inf,d)
     if(is.null(ub)) ub <- rep(Inf,d)
     if (BOUNDS) {
@@ -40,12 +49,8 @@ function (Explore, n = 1000, replace.target = c(0, 1, 2),
               ub <- rep(ub, d)
       if ((length(lb) != d) || (length(lb) != d)) 
           stop("lb, ub incorrect dimension")
-      if (is.na(sum(lb + ub))) 
-          stop("hybrid.explore: NaN lb, ub")
-      cat("hybrid.sample: Placing bounds lb =\n")
-      cat(lb, "\n")
-      cat("hybrid.sample: Placing bounds ub =\n")
-      cat(ub, "\n")
+      if (is.na(sum(abs(lb)+abs(ub))))
+          stop("NaN for lb, ub")
     }
     W <- function(p) return(sum(p^2)/2)
     if (!replace.target[1] %in% 0:2) 
@@ -66,10 +71,6 @@ function (Explore, n = 1000, replace.target = c(0, 1, 2),
     x <- X_select[indx_best, ]
     fx <- y_select[indx_best]
     if(is.null(dim(x))) x <- t(x)
-    cat("\n")
-    cat("hybrid.sample: L =", L, "and delta =", delta, "\n")
-    cat("hybrid.sample: Iterations n =", n, "with", nchains, 
-        "chains: Tj =", Temp, "\n")
     fcalls <- 0
     x1 <- x[1, ]
     fx1 <- fx[1]
@@ -88,15 +89,8 @@ function (Explore, n = 1000, replace.target = c(0, 1, 2),
         KEY.VARS <- rbind(KEY.VARS, pair)
     }
     if (graph) {
-        if(!exists(".Rpackage_DEMO")) {
-          Rpackage_DEMO <- .Rpackage_DEMO <- FALSE
-        } else {
-          Rpackage_DEMO <- .Rpackage_DEMO
-          assign(".Rpackage_DEMO", FALSE, envir=.GlobalEnv)
-        }
-        nrefresh.graph <- ceiling(10/nchains)
-        if(is.logical(Rpackage_DEMO))
-          if(Rpackage_DEMO) nrefresh.graph <- Inf
+        if(dim(X)[2] <= 2) nrefresh.graph <- Inf else
+          nrefresh.graph <- ceiling(10/nchains)
         graph.template <- .hybrid.template
         count <- key.indx <- 1
         if (is.null(graph.template)) 
@@ -122,8 +116,6 @@ function (Explore, n = 1000, replace.target = c(0, 1, 2),
     else Rank <- rank(Temp)
     for (chain in 1:nchains) {
         path <- Rank[chain]
-        cat(sep = "", "x(", chain, ", T =", Temp[chain], ") =\n")
-        cat(rnd(x[chain, ]), "\n")
         if (graph) 
             points(x[chain, keyvars[1]], x[chain, keyvars[2]], 
                 pch = 19, col = clrs[path])
@@ -156,7 +148,7 @@ function (Explore, n = 1000, replace.target = c(0, 1, 2),
         c.vec_x <- c.vec(x)
         val <- lsq - as.double(t(c.vec_x) %*% Sigma.inv %*% c.vec_x)
         if (is.na(val)) {
-            cat("hybrid.sample: NaN sigmaf(", rnd(x), ")\n")
+            warning("hybrid.sample: NaN sigmaf(", rnd(x), ")\n")
             return(Inf)
         }
         if (val < 0) {
@@ -171,38 +163,17 @@ function (Explore, n = 1000, replace.target = c(0, 1, 2),
         }
         return(sqrt(val))
     }
-    cat("hybrid.sample: sd(f(x))/lambda:\n");
     Ey <- cory <- NULL
     for(i in 1:nexplore) {
       Ey <- c(Ey, Ef(X[i,]))
       cory <- c(cory, sigmaf(X[i,])/sqrt(lsq))
     }
-    cat("E(f(x)):\n")
-    print(summary(Ey))
-    cat("sd(f(x)):\n")
-    print(summary(cory))
     if (is.null(maxleap)) maxleap <- max(cory)
     SAMP <- ySAMP <- NULL
     its <- 1
     acceptance <- NULL
     swaps.record <- swaps.record1 <- NULL
     repeat {
-        cat("\n")
-        if (nchains == 1) {
-            cat("hybrid.sample: Single chain used ")
-            if (replace.target[1] == 0) 
-                cat("using true target\n")
-            else cat("with target replaced\n")
-        }
-        else {
-            cat("Parallel Tempering with", nchains, "chains and T =", 
-                Temp, "\n")
-            if (replace.target[1] == 1) 
-                cat("hybrid.sample(replace.target=1): Replace target in chains with T>1\n")
-            else if (replace.target[1] == 2) 
-                cat("hybrid.sample(replace.target=2): Replace target in all chains\n")
-            else cat("hybrid.sample: Using true target\n")
-        }
         if (nchains >= 3) 
             for (reps in 1:nswaps) for (index1 in sample(2:(nchains - 
                 1))) {
@@ -228,8 +199,6 @@ function (Explore, n = 1000, replace.target = c(0, 1, 2),
                 if (log(runif(1)) < alpha) {
                   Temp[swap[1]] <- T2
                   Temp[swap[2]] <- T1
-                  cat("hybrid.sample: Swap T =", T1, "with", 
-                    T2, "\n")
                   swaps.record <- c(swaps.record, 1)
                 }
                 else {
@@ -268,23 +237,18 @@ function (Explore, n = 1000, replace.target = c(0, 1, 2),
                 mean.acceptance <- 0
                 if (!is.null(acceptance))
                   mean.acceptance <- mean(acceptance)
-                cat(sep = "", "Sampling x(T=1) of ", nchains, 
-                  " chains: ", length(acceptance), " / ", n, " iterations with ", 
-                  round(10000 * mean.acceptance)/100, "% acceptance\n")
                 if (is.na(fswap2)) 
                   stop("f(x) NaN")
                 if (fswap2 == Inf) 
                   stop(paste("Swaps: Infinite f(x) not allowed:\n", 
                     "f(", deparse(xswap2), ") =", fswap2))
-                cat("Swaps: f(x) =", fswap2, "\n")
                 fcalls <- fcalls + 1
                 if (graph) 
-                  .show.time(date1, date2, graph)
+                  .show.time(date1, date2)
             }
             alpha <- ((fswap2 - fswap1)/1 + (Efswap1 - Efswap2)/T2)
             if(is.na(alpha)) alpha <- -Inf
             if (log(runif(1)) < alpha) {
-                cat("hybrid.sample: Swap T = 1 with", T2, "\n")
                 Temp[swap[1]] <- T2
                 Temp[swap[2]] <- 1
                 swaps.record1 <- c(swaps.record1, 1)
@@ -293,11 +257,8 @@ function (Explore, n = 1000, replace.target = c(0, 1, 2),
                   fx1 <- fswap2
                 }
             }
-            else {
-                cat("hybrid.sample: Reject swap between T = 1 &", 
-                  T2, "\n")
+            else
                 swaps.record1 <- c(swaps.record1, 0)
-            }
         }
         if (T.mult == 1) 
             Rank <- 1:nchains
@@ -306,10 +267,6 @@ function (Explore, n = 1000, replace.target = c(0, 1, 2),
             path <- Rank[chain]
             x.C <- x[chain, ]
             p.C <- NULL
-            cat("\n")
-            cat(sep = "", "Leapfrog from x(", chain, ", T = ", 
-                Temp[chain], "):\n")
-            cat("", rnd(x.C), "\n")
             if (replace.target[1] == 0) 
                 fx.C <- fx[chain]
             else if ((replace.target[1] == 1) && (path == 1)) 
@@ -322,8 +279,18 @@ function (Explore, n = 1000, replace.target = c(0, 1, 2),
                 xOK <- FALSE
                 delta_adj <- delta
                 try({
-                  xp_Early <- .Call("Leapfrog", X, y, x.C, p.C, 
-                    L, delta_adj, GPfit$Sigma.inv, params, Temp[chain], Inf, lb, ub)
+                  out <- .C("Leap", as.double(X), as.integer(dim(X)[1]),
+                    as.integer(d), as.double(y), as.double(x.C),
+                    as.double(p.C), as.integer(L), as.double(delta),
+                    as.double(GPfit$Sigma.inv), as.double(params),
+                    as.double(Temp[chain]), as.double(sqrt(lsq)*maxleap),
+                    as.double(low), as.double(upp),
+                    as.double(rep(0,n)), as.double(rep(0,n)),
+                    as.double(rep(0,d)), as.double(rep(0,d)),
+                    xp_Early = as.double(xp_Early) )
+                  xp_Early <- out$xp_Early
+                  dim(xp_Early) <- c(d,3)
+                  xOK <- TRUE
                   xOK <- TRUE
                 })
                 xout <- xp_Early[, 1]
@@ -331,15 +298,11 @@ function (Explore, n = 1000, replace.target = c(0, 1, 2),
                   xOK <- FALSE
                 if (xOK) 
                   break
-                cat("hybrid.sample: Proposed x failed:\n", xout, "\n")
             }
             out <- list(x = xp_Early[, 1], p = xp_Early[, 2])
             H_xp.C <- -fx.C + W(p.C)
             x.P <- out$x
             p.P <- out$p
-            cat(sep = "", "Proposed x(", chain, ", T =", 
-              Temp[chain], "):\n")
-            cat("", rnd(x.P), "\n")
             if (graph) 
               lines(c(x.C[keyvars[1]], x.P[keyvars[1]]), 
                 c(x.C[keyvars[2]], x.P[keyvars[2]]), lwd = 0.5, 
@@ -349,7 +312,6 @@ function (Explore, n = 1000, replace.target = c(0, 1, 2),
             else if ((replace.target[1] == 1) && (path > 1)) 
               fx.P <- Ef(x.P)
             else {
-              cat("hybrid.sample: Calculating true f(x) ..\n")
               date1 <- date()
               fx.P <- f(x.P)
               date2 <- date()
@@ -359,17 +321,14 @@ function (Explore, n = 1000, replace.target = c(0, 1, 2),
                 stop(paste("Leapfrog: Infinite f(x) not allowed:\n", 
                   "f(", deparse(x.P), ") =", fx.P))
               fcalls <- fcalls + 1
-              cat("f(x) =", fx.P, "with leap from f(x) =", 
-                fx.C, "\n")
               if (graph) 
                 if (path == 1) 
-                  .show.time(date1, date2, graph)
+                  .show.time(date1, date2)
             }
             H_xp.P <- -fx.P + W(p.P)
             if ((replace.target[1] == 2) || ((replace.target == 1) && (path > 1))) {
               corf_x.C <- sigmaf(x.C)/sqrt(lsq)
               corf_x.P <- sigmaf(x.P)/sqrt(lsq)
-              cat("sd(f(x))/lambda =", corf_x.P, "(maxleap =", maxleap, "\b)\n")
               H_xp.C <- H_xp.C + r * (corf_x.C - maxleap)^2/(1 - corf_x.C)^2
               H_xp.P <- H_xp.P + r * (corf_x.P - maxleap)^2/(1 - corf_x.P)^2
             }
@@ -382,21 +341,23 @@ function (Explore, n = 1000, replace.target = c(0, 1, 2),
               else if (replace.target[1] == 1) 
                 if (path == 1) 
                   fx1 <- fx.P
-              cat("ACCEPT x.P(", chain, ")\n")
               if (path == 1) {
                 SAMP <- rbind(SAMP, x.P)
                 ySAMP <- c(ySAMP, fx.P)
                 acceptance <- c(acceptance, 1)
+                if(dim(SAMP)[1] %% floor(n/10) == 0)
+                  message(paste(sep="","hybrid.sample: ",
+                    round(100*dim(SAMP)[1]/n),"% Complete with ",
+                    round(100*mean(acceptance),1),"% acceptance of proposals"))
               }
               if (graph) 
                 points(x.P[keyvars[1]], x.P[keyvars[2]], 
                   pch = 20, col = clrs[path])
             }
             else {
-              if (graph) if(!Rpackage_DEMO)
+              if (graph)
                 points(x.P[keyvars[1]], x.P[keyvars[2]], 
                   pch = "X", col = reds[path])
-              cat("REJECT x.P(", chain, ")\n")
               if (path == 1) {
                 SAMP <- rbind(SAMP, x.C)
                 ySAMP <- c(ySAMP, fx.C)
@@ -406,14 +367,6 @@ function (Explore, n = 1000, replace.target = c(0, 1, 2),
             mean.acceptance <- 0
             if (!is.null(acceptance))
               mean.acceptance <- mean(acceptance)
-            cat(sep = "", "Sampling x(T=1) of ", nchains, 
-              " chains: ", length(acceptance), " / ", n, " iterations with ", 
-              round(10000 * mean.acceptance)/100, "% acceptance\n")
-            cat("Leapfrog used: L =", L, ", delta =", delta_adj, "\n")
-            if (nchains >= 2) 
-              cat(sep = "", "Swaps (T=1 with T>1) ", round(10000 * mean(swaps.record1))/100, "%\n")
-            if (nchains >= 3) 
-              cat(sep = "", "Swaps (T>1) ", round(10000 * mean(swaps.record))/100, "%\n")
         }
         if (graph) {
             if (count >= nrefresh.graph) {
@@ -432,8 +385,6 @@ function (Explore, n = 1000, replace.target = c(0, 1, 2),
             count <- count + 1
         }
         its <- its + 1
-        assign("hybrid.sample.out", list(SAMP = SAMP, y = ySAMP, x=x, fx=fx,
-                Temp=Temp, GPfit=GPfit, acceptance = acceptance, function.calls = fcalls), envir = .GlobalEnv)
         if (length(acceptance) > n) 
             break
     }
